@@ -5,7 +5,7 @@ These are fixtures that every app needs so probably you will not be editing this
 import os
 import sys
 import logging
-from py4web import Session, Cache, Translator, DAL, Field
+from py4web import Session, Cache, Translator, Flash, DAL, Field
 from py4web.utils.mailer import Mailer
 from py4web.utils.auth import Auth
 from py4web.utils.tags import Tags
@@ -28,15 +28,18 @@ for item in settings.LOGGERS:
     logger.addHandler(handler)
 
 # connect to db
-db = DAL(settings.DB_URI,
-         folder=settings.DB_FOLDER,
-         pool_size=settings.DB_POOL_SIZE,
-         migrate = settings.DB_MIGRATE,
-         fake_migrate= settings.DB_FAKE_MIGRATE)
+db = DAL(
+    settings.DB_URI,
+    folder=settings.DB_FOLDER,
+    pool_size=settings.DB_POOL_SIZE,
+    migrate=settings.DB_MIGRATE,
+    fake_migrate=settings.DB_FAKE_MIGRATE,
+)
 
 # define global objects that may or may not be used by th actions
 cache = Cache(size=1000)
 T = Translator(settings.T_FOLDER)
+flash = Flash()
 
 # pick the session type that suits you best
 if settings.SESSION_TYPE == "cookies":
@@ -47,7 +50,11 @@ elif settings.SESSION_TYPE == "redis":
     host, port = settings.REDIS_SERVER.split(":")
     # for more options: https://github.com/andymccurdy/redis-py/blob/master/redis/client.py
     conn = redis.Redis(host=host, port=int(port))
-    conn.set = lambda k, v, e, cs=conn.set, ct=conn.ttl: cs(k, v, ct(k)) if ct(k) >= 0 else cs(k, v, e)
+    conn.set = (
+        lambda k, v, e, cs=conn.set, ct=conn.ttl: cs(k, v, ct(k))
+        if ct(k) >= 0
+        else cs(k, v, e)
+    )
     session = Session(secret=settings.SESSION_SECRET_KEY, storage=conn)
 elif settings.SESSION_TYPE == "memcache":
     import memcache, time
@@ -63,7 +70,7 @@ auth = Auth(session, db, define_tables=False)
 auth.use_username = True
 auth.registration_requires_confirmation = settings.VERIFY_EMAIL
 auth.registration_requires_approval = settings.REQUIRES_APPROVAL
-auth.allowed_actions = ['all']
+auth.allowed_actions = ["all"]
 auth.login_expiration_time = 3600
 auth.password_complexity = {"entropy": 50}
 auth.block_previous_password_num = 3
@@ -75,7 +82,8 @@ if settings.SMTP_SERVER:
         sender=settings.SMTP_SENDER,
         login=settings.SMTP_LOGIN,
         tls=settings.SMTP_TLS,
-        ssl=settings.SMTP_SSL)
+        ssl=settings.SMTP_SSL,
+    )
 
 if auth.db:
     groups = Tags(db.auth_user, "groups")
@@ -137,5 +145,5 @@ if settings.USE_CELERY:
 # the template, although we recommend client-side translations instead
 auth.enable(uses=(session, T, db), env=dict(T=T))
 
-unauthenticated = ActionFactory(db, session, T, auth)
-authenticated = ActionFactory(db, session, T, auth.user)
+unauthenticated = ActionFactory(db, session, T, flash, auth)
+authenticated = ActionFactory(db, session, T, flash, auth.user)
